"""
    Dummy conftest.py for cuebung_helper.

    If you don't know what this is for, just leave it empty.
    Read more about conftest.py under:
    https://pytest.org/latest/plugins.html
"""
from __future__ import print_function, absolute_import, division

import pytest  # pylint: disable=W0611
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3
